'''
Gizmo(QP) Web Framework - http://gizmojo.org/
------------------------------------------------------------------------------
Copyright (C) 2007 Mario Ruggier, mario@ruggier.org
Licensed under the Open Software License version 3.0
'''
__url__ = "$URL: svn://gizmojo.org/pub/gz/trunk/fill/admin_users.py $"
__revision__ = "$Id: admin_users.py 993 2008-03-21 17:21:23Z mario $"

from qpy import h8
from gz.pub.common import get_publisher, get_request, get_dhi, page
from gz.gizmo.stbl import Stbl
from gz.gizmo.gform import GForm
from gz.gizmo.gspec import Spec
from gz.gizmo.gspec import pattern, both, length

from directory import Directory
from exportable import X
from gz.skin import html

spec_hints = dict(
    userid = (
        Spec(both(
            Spec(pattern('^[a-zA-Z_][a-zA-Z0-9_]*$'), 
                'Only alphanumeric _- and cannot start with a number.'),
            Spec(length(3, None), 'At least 3 characters.')),
            'Malformed... ', 'Userid is correctly formed.'),
        ('Only alphanumeric _- and cannot start with a number. '
         'At least 3 characters. Lowercase (case-insensitive).'),
    ),
    email = (
        Spec(pattern('^.+@.+\..{2,4}$'),
            'Email is malformed.', 'Email is correctly formed.'),
        'Valid email address, and not used by another user.',
    ),
    password = (
        Spec(length(4, None), 'Too short.', 'Password is OK.'),
        # Spec(both(
        #       Spec(pattern('^\d|^\D+\d'), 'at least 1 digit.'),
        #       Spec(length(4, None), 'too short.')),
        #    'Password is not OK... ', 'Password is OK.')),
        None,
    ),
)

def get_spechint(key):
    ''' (key:str) -> (spec, hint) '''
    publisher = get_publisher()
    if publisher.has_key_path('spec_hints', key):
        return publisher.configuration['spec_hints'][key]
    else:
        return spec_hints[key]


class UserDir(Directory):
    
    exportables = []
    
    def _q_lookup(self, component):
        if component == '_':
            component = ''
        user = get_publisher().get_users().get(component)
        # X.attribute=None forces lookup via _q_lookup(), that in turn
        # ensures self.exportables is reset on each successful lookup.
        if user is not None:
            uid = user.id 
            self.exportables = [ X(uid, None, uid, uid, slashed=True) ]
            return UserUI(user)

class UserUI(Directory):

    exportables = [
        X('', 'index', None, 'View User', ),
        X('view', 'view', 'View', 'View User', ),
        X('modify', 'modify', 'Modify', 'Modify User', ),
        X('passwds', 'passwds', 'Passwords', 'Passwords', ),
        X('permissions', 'permissions', 'Permissions', 'Permissions', ),
    ]

    def __init__(self, user):
        self.user = user

    def index(self):
        get_publisher().redirect('view')

    def view(self):
        publisher, dhi = get_publisher(), get_dhi()
        def define_form(gizmoid, dhi):
            form = GForm(gizmoid, 
                        dhi=dhi, 
                        use_tokens=True, 
                        check_evt='onchange')
            form.add_string('userid', title="Username", 
                value=self.user.get_id(), 
                readonly='readonly')
            form.add_string('email', title="Email", 
                required=True,
                value=self.user.get_email(),
                readonly='readonly')
            return form
        form = define_form('u', dhi)
        return page('User: %s' %(self.user.get_id() or 'Null User'), 
            form.render(), 
            template='admin')

    def passwds(self):
        publisher, dhi = get_publisher(), get_dhi()
        def define_form_reset(gizmoid, dhi):
            form = GForm(gizmoid, 
                        dhi=dhi, 
                        use_tokens=False, 
                        check_evt='onchange')
            form.add_singleselect('realm', required=True, 
                title='User Realms',
                options=self.user.digester.get_realms() or [''])
            passwd_spec, passwd_hint = get_spechint('password')
            form.add_password('passwd', title="Password", 
                required=True, 
                hint=passwd_hint,
                spec=passwd_spec)
            form.add_submit('reset', value='Reset', check_evt=False)
            form.add_submit('remove', value='Remove', check_evt=False)
            return form
        def define_form_add(gizmoid, dhi):
            form = GForm(gizmoid, 
                        dhi=dhi, 
                        use_tokens=False, 
                        check_evt='onchange')
            readonly = None
            if not other_realms:
                readonly = "readonly"
            form.add_singleselect('realm_add', required=True, 
                title='Other Realms',
                options=other_realms or [''],
                readonly=readonly)
            passwd_spec, passwd_hint = get_spechint('password')
            form.add_password('passwd_add', title="Password", 
                required=True, 
                hint=passwd_hint,
                spec=passwd_spec,
                readonly=readonly)
            form.add_submit('add', value='Add', check_evt=False)
            return form
        def handle_passwd_realm(form, passwd_name, realm_name):
            if form.is_submitted() and not form.has_errors():
                passwd = form.get(passwd_name)
                realm = form.get(realm_name)
                sbmt = form.get_submit()
                if sbmt == 'remove':
                    print 'Removing (%s, %s) passwd ... ' % (
                                                self.user.get_id(), realm)
                    self.user.get_digester().remove_digest(realm)
                    publisher.redirect('')
                elif sbmt=='reset' or sbmt=='add':
                    if not self.user.has_password(passwd, realm):
                        print 'Changing (%s) passwd... ' % (
                                                self.user.get_id())
                        self.user.set_password(passwd, realm)
                    publisher.redirect('')
        form_reset = define_form_reset('pwr', dhi)
        other_realms = [ realm for realm in publisher.get('realms', []) 
                         if realm not in self.user.digester.get_realms() ]
        form_add = define_form_add('pwa', dhi)
        handle_passwd_realm(form_reset, 'passwd', 'realm')
        handle_passwd_realm(form_add, 'passwd_add', 'realm_add')
        return page('User Passwds: %s' %(self.user.get_id() or 'Null User'), 
            h8.from_list((
                form_reset.render(), form_add.render())), 
            template='admin')
    
    def permissions(self):
        publisher, dhi = get_publisher(), get_dhi()
        #+ Only handles domain-wide permission, i.e. granter is True.
        user_perms = [ perm for perm in self.user.permissions.keys()
                       if self.user.permissions[perm] ]
        other_perms = [ perm for perm in publisher.get('permissions') 
                        if perm not in user_perms ]
        def define_form_userperms(gizmoid, dhi):
            form = GForm(gizmoid, 
                        dhi=dhi, 
                        use_tokens=False, 
                        check_evt='onchange')
            form.add_singleselect('perm_rm', required=True, 
                title='User Granted Permissions',
                options=user_perms or [''])
            form.add_submit('remove', value='Remove', check_evt=False)
            return form
        def define_form_otherperms(gizmoid, dhi):
            form = GForm(gizmoid, 
                        dhi=dhi, 
                        use_tokens=False, 
                        check_evt='onchange')
            form.add_singleselect('perm_add', required=True, 
                title='Other Permissions',
                options=other_perms or [''])
            form.add_singleselect('perm_add_qual', 
                title='Restriction / Extension',
                options=['', 'traverse', 'create', 'update', 'delete'])
            form.add_submit('add', value='Add', check_evt=False)
            return form
        def handle_permission(form, perm_name):
            if form.is_submitted() and not form.has_errors():
                perm = form.get(perm_name)
                sbmt = form.get_submit()
                if sbmt=='remove':
                    print 'UNGRANTING: user=%s permission=%s granter=%s' % (
                                                self.user.get_id(), perm, True)
                    self.user.permissions.ungrant(perm, True)
                    publisher.redirect('')
                elif sbmt=='add': 
                    restriction = form.get('perm_add_qual')
                    if restriction:
                        perm = '%s_%s' % (perm, restriction)
                    print 'GRANTING: user=%s permission=%s granter=%s' % (
                                                self.user.get_id(), perm, True)
                    self.user.permissions.grant(perm, True)
                    publisher.redirect('')
        # self.user.permissions.data.items()
        form_userperms = define_form_userperms('pru', dhi)
        form_otherperms = define_form_otherperms('pro', dhi)
        handle_permission(form_userperms, 'perm_rm')
        handle_permission(form_otherperms, 'perm_add')
        return page('User Permissions: %s' % ( 
                                        self.user.get_id() or 'Null User'),
            h8.from_list((
                form_userperms.render(), form_otherperms.render())), 
            template='admin')
    
    def modify(self):
        publisher, dhi = get_publisher(), get_dhi()
        #+ delete
        def define_form(gizmoid, dhi):
            form = GForm(gizmoid, 
                        dhi=dhi, 
                        use_tokens=True, 
                        check_evt='onchange')
            form.add_string('userid', title="Username", 
                value=self.user.get_id(), 
                readonly='readonly')
            email_spec, email_hint = get_spechint('email')
            form.add_string('email', title="Email", 
                required=True,
                value=self.user.get_email(),
                hint=email_hint,
                spec=email_spec,
                onblur='javascript:this.value=this.value.toLowerCase();')
            form.add_submit('modify', value='Modify', check_evt=False)
            return form
        form = define_form('mu', dhi)
        if form.is_submitted() and not form.has_errors():
            # ok, proceed
            email = form.get('email')
            if email != self.user.get_email():
                print 'Changing (%s) email from %s to %s' % (
                        self.user.get_id(), self.user.get_email(), email)
                self.user.email = email
            publisher.redirect('.')
        return page('Modify User: %s' %(self.user.get_id() or 'Null User'), 
            form.render(),
            template='admin')

        
class AdminUsers(Directory):

    exportables = [
        X('', 'index', 'Users', 'Manage Users', ),
        X('new', 'new_user', 'New', 'New User', ),
        X('user/', 'existing_user', None, 'User', ),
    ]

    def _q_lookup(self, component):
        # Very similar to RegisterDir
        if component.endswith('.json'):
            field_name = component[:-5]
            get_publisher().json_callback_fs(self, field_name)
            
    def index(self):
        publisher, dhi = get_publisher(), get_dhi()
        # list users/email/permissions
        user_data = [ (
            html.href('user/%s/modify'%(user_id or '_'), user_id or 'Null User'), 
            user.get_email(), 
            ' '.join([ perm for perm in user.permissions.keys()
                       if user.is_granted(perm, True) ]) or ' '
                    )
                    for user_id, user 
                    in publisher.get_users().items() 
            ]
        u = Stbl('s', dhi=dhi, col_types=['str', 'str', 'str',], 
                    sort_on=0, sort_sense='desc')
        u.set('width', '80%')
        u.set('status', False) 
        u.set('columns', [ # [ (name, title) ] 
                    ('User ID', 'User ID'), 
                    ('Email', 'Email'), 
                    ('Permissions', 'Permissions'), ])
        u.set('column_fcodes', ('%s', '%s', '%s') )
        u.set('rows_data', user_data)
        u.set_style('s', 'max-width', '80%')
        u.set_style('s', 'margin', '1em 0')
        return page(dhi.x.title, 
            u.render(),
            template='admin')

    existing_user = UserDir()
        
    def new_user(self):
        publisher, dhi = get_publisher(), get_dhi()
        form = self.define_new_user_form('nu', dhi)
        if form.is_submitted() and not form.has_errors():
            # ok, proceed
            userid = form.get('userid')
            u = publisher.get_users()[userid] = publisher.create_user(userid)
            u.email = form.get('email')
            u.set_password(form.get('passwd'), publisher.get_default_realm())
            publisher.redirect('./user/%s/'%(userid))
        return page(dhi.x.title,
            form.render(),
            template='admin')

    # Very similar to RegisterDir: 
    
    def define_new_user_form(self, gizmoid, dhi):
        form = GForm(gizmoid, dhi=dhi, use_tokens=True, check_evt='onchange')
        userid_spec, userid_hint = get_spechint('userid')
        form.add_string('userid', title="Username", 
            required=True, 
            hint=userid_hint,
            spec=userid_spec,
            callback='json',
            onblur='javascript:this.value=this.value.toLowerCase();')
        email_spec, email_hint = get_spechint('email')
        form.add_string('email', title="Email", 
            required=True,
            hint=email_hint,
            spec=email_spec,
            onblur='javascript:this.value=this.value.toLowerCase();')
        #+ realm
        passwd_spec, passwd_hint = get_spechint('password')
        form.add_password('passwd', title="Password", 
            required=True, 
            hint=passwd_hint,
            spec=passwd_spec)
        form.add_submit('add', value='Add', check_evt=False)
        return form

    _userid_attr_name = 'id'
    def callback_fs_userid(self, fs):
        publisher, value = get_publisher(), fs['value']
        User = publisher.get_user_class()
        user = publisher.get_users().get(value)
        if not publisher.match(value, User, self._userid_attr_name):
            fs['ok'] = False
            fs['message'] = 'Sorry, need a valid user name.'
        elif user:
            fs['ok'] = False
            fs['message'] = 'Sorry, name is taken.'
        else:
            fs['message'] = 'OK, name is available.'
        return fs

    # End very similar to RegisterDir
    
