'''
Gizmo(QP) Web Framework - http://gizmojo.org/
------------------------------------------------------------------------------
Copyright (C) 2007 Mario Ruggier, mario@ruggier.org
Licensed under the Open Software License version 3.0
'''
__url__ = "$URL: svn://gizmojo.org/pub/gz/trunk/fill/dhi.py $"
__revision__ = "$Id: dhi.py 993 2008-03-21 17:21:23Z mario $"

from gz.lib.design_patterns import bunch

class Dhi(object):
    ''' DHTMLInfo type

    Any functionality participating in building the response for a hit may
    build up this object as is necessary for its needs.

    A response that uses a Dhi must always work either on a copy or on a
    newly created instance, obtained by calling gz.pub.common.get_dhi().
    The Dhi instance is then cached on publisher.hit.info['dhi'], during
    the hit's processing.

    File URL paths should be specified either relative to SiteDirectory's 
    "assets" export (site needs to define it) or as root-relative (starts 
    with a slash). In both cases, the script name is automatically added
    as needed. 
    '''

    def __init__(self, 
            css_files=None,         # [ either(str:str, attrs:dict ]
            css_codes=None,         # [ code:str ]
            js_files=None,          # [ either(str:str, attrs:dict ]
            js_codes=None,          # [ code:str ]
            onload_actions=None,    # [ code:str ]
            body_attrs=None,        # {}
            gizmos=None,            # bunch
            meta=None,              # bunch
            http_equiv=None         # bunch
        ):

        # Clients may assume that these attributes are not None
        self.css_files = css_files or []
        self.css_codes = css_codes or []
        self.js_files = js_files or []
        self.js_codes = js_codes or []
        self.onload_actions = onload_actions or []
        self.body_attrs = body_attrs or {}
        self.gizmos = gizmos or bunch()

        # These attributes may be None (often unused)
        self.meta = meta # None | bunch 
        self.http_equiv = http_equiv # None | bunch 

        # In working instances this is a reference to parent exportable
        self.x = None

    def prepend_css_file(self, css_file):
        if css_file not in self.css_files:
            self.css_files.insert(0, css_file)

    # css_file is either(src:str, attrs:dict)
    def add_css_files(self, *css_files):
        for css_file in css_files:
            if css_file not in self.css_files:
                self.css_files.append(css_file)

    def add_css_codes(self, *css_codes):
        for css_code in css_codes:
            if css_code not in self.css_codes:
                self.css_codes.append(css_code)

    # js_file is either(src:str, attrs:dict)
    def add_js_files(self, *js_files):
        for js_file in js_files:
            if js_file not in self.js_files:
                self.js_files.append(js_file)

    def add_js_codes(self, *js_codes):
        for js_code in js_codes:
            if js_code not in self.js_codes:
                self.js_codes.append(js_code)

    def add_onload_actions(self, *onload_actions):
        for onload_action in onload_actions:
            if onload_action not in self.onload_actions:
                self.onload_actions.append(onload_action)

    # meta, http_equiv may be None
    def add_meta(self, name, value):
        if self.meta is None:
            self.meta = bunch()
        self.meta[name] = value
    def add_http_equiv(self, name, value):
        if self.http_equiv is None:
            self.http_equiv = bunch()
        self.http_equiv[name] = value
    
