'''
Gizmo(QP) Web Framework - http://gizmojo.org/
------------------------------------------------------------------------------
Copyright (C) 2007 Mario Ruggier, mario@ruggier.org
Licensed under the Open Software License version 3.0
'''
__url__ = "$URL: svn://gizmojo.org/pub/gz/trunk/fill/exportable.py $"
__revision__ = "$Id: exportable.py 971 2008-03-18 19:49:42Z mario $"

from gz.lib.design_patterns import bunch

class X(bunch):
    ''' eXportable type
    Dictionary-like data object to specify an exportable. 
    X(
        # External path_component
        # Trailing slash -> shorthand for X.slashed=True
        # "abs" and None attribute -> resource is external, see is_external()
        component: str,
        
        # Internal attribute_name
        # None -> resource for this exportable is via _q_lookup()
        # None AND "abs" component -> resource is external, see is_external()
        attribute: str | None,

        # Used as link label to this exportable
        # None means should not be included in menu links
        crumb: str | None, 

        # Used as title attribute of links, and as page title
        #+ Clarify parent (no link) / child (link) scenario
        title: str | None,
            
        # Access control tuple
        # role_permission:str, 
        #    either predefined roles:
        #       'anonymous'         user not logged in
        #       'authenticated'     user logged in 
        #       'callback'          service url only (e.g. ajax callback)
        #       'administrator'     user is admin, and logged in
        #   or any application-specific permission
        #
        allow: either(
                sequence(role_permission:str, tuple), # always OR-combined
                None,  # no access restrictions (inherit)
                True,  # explicit allow
                False, # explicit deny
        )

        # dhtml_info
        dhi: dhi
        
        # slashed: to avoid unnecessary redirection for menu items
        slashed: bool
    )
    '''
    
    def __init__(self, component, attribute, crumb, title, 
                    allow=None, dhi=None, slashed=False, **kw): 
        if component.endswith('/'):
            component = component[:-1]
            slashed = True
        super(X, self).__init__(
            component=component, 
            attribute=attribute, 
            crumb=crumb, 
            title=title,
            allow=allow, 
            dhi=dhi,
            slashed=slashed,
            **kw)
    
    def to_4tuple(self):
        ''' () -> (comp, attr, crumb, title) 
        Returns the 4-tuple version of the export, expected by qp.
        '''
        return (self.component, self.attribute, self.crumb, self.title)

    def is_external(self):
        ''' () -> bool
        The convention to mark an "exportable" as an external link is:
        a) The (external) component is "absolute"
        b) The (internal) attribute is None
        '''
        if self.attribute is None:
            if self.component.find('://')>3:
                return True
            if self.component.startswith('/'):
                return True
        return False

#
