'''
Gizmo(QP) Web Framework - http://gizmojo.org/
------------------------------------------------------------------------------
Copyright (C) 2007 Mario Ruggier, mario@ruggier.org
Licensed under the Open Software License version 3.0
'''
__url__ = "$URL: svn://gizmojo.org/pub/gz/trunk/fill/login_dir.py $"
__revision__ = "$Id: login_dir.py 975 2008-03-19 10:44:16Z mario $"

from qpy import h8
from qp.fill.form import Form
from gz.pub.common import get_publisher, get_request, get_session, get_dhi
from gz.fill.directory import Directory
from gz.fill.exportable import X


class LoginDir(Directory):

    exportables = [
        X('', 'login', 'Login', 'Login', ('anonymous',)),
        X('logout', 'logout', None, 'Logout'),
    ]

    def login(self):
        get_publisher().ensure_signed_in()

    def logout(self):
        publisher, request = get_publisher(), get_request()
        back_to = request.environ.get('HTTP_REFERER', '/')
        if back_to == publisher.complete_url(request.get_path()):
            back_to = '/'
        publisher.sign_out(back_to)

    #
    
    def get_t(self):
        def t(s):
            return s
        return t

    def login_form(self):
        session = get_session()
        default_user_id = None
        if session.get_owner():
            default_user_id = session.get_owner().get_id()
        t = self.get_t()
        form = Form(use_tokens=True)
        form.add_string('name', id='fname', 
            value=default_user_id, 
            title=t('Username'), 
            required=False,
            onblur='javascript:this.value=this.value.toLowerCase();')
        form.add_password('password', id='fpassword', 
            title=t('Password'), 
            required=False)
        form.add_submit('login', t('Sign in'))
        return form 
    
    def respond_login_form(self, title, form, **kw):
        publisher, request, dhi = get_publisher(), get_request(), get_dhi()
        # The login "page", or screen, is special, as it may be displayed for 
        # any url - we remember that the login screen is being shown: 
        publisher.get_hit().get_info()['login_screen'] = True
        t = self.get_t()
        if form.is_submitted():
            userid = form.get('name')
            if userid is not None:
                userid.lower()
            user = publisher.get_users().get(userid)
            if not user:
                form.set_error('name', t('unknown user'))
            #elif not user.is_abled():
            #    form.set_error('name', 'user is not activated')
            else:
                form.set_error('password', t('wrong password'))      
        dhi.add_css_files('/gza/gform.css') # uses some of the same CSS rules
        dhi.add_onload_actions(h8('document.getElementById("fname").focus();'))
        publisher.respond(t(dhi.x.title),
                    self.login_content(form), 
                    template='login', 
                    **kw)

    def login_content(self, form):
        return form.render()
        
    #
