/* 
Gizmo(QP) Web Framework - http://gizmojo.org/
------------------------------------------------------------------------------
Copyright (C) 2007 Mario Ruggier
Licensed under the Open Software License version 3.0
------------------------------------------------------------------------------
$Id: stbl.js 964 2008-03-17 14:03:29Z mario $ 
*/

// ---------------------------------------------------------------------------
// SortableTable - constructor and methods 

function Stbl(gizmoid, col_types, sortOn, sortSense) {
    this.gizmoid = gizmoid;
    this.col_types = col_types; 
    this.sortOn = parseInt(sortOn); 
    this.prevSortOn = this.sortOn; 
    this.sortSense = sortSense; 
    this.sorting = false; 
    this.num_rows = 51; // > pagesize, reset on first sort. 
    this.onBeforeSort = null;
    this.onAfterSort = null;
    this.stable = null; 
    this.thead = null; 
    this.base_assets = null;
}
Stbl.prototype = new Gizmo;
Stbl.prototype.elem_names = ['status_first', 'status_runin', 'status_main'];
Stbl.prototype.event_handlers = { 'sort': null };

Stbl.prototype.onload = function () {
    this.stable = this.get_elem(this.gizmoid); // table dom element
    this.thead = this.stable.getElementsByTagName('thead')[0];
    var hcols = this.thead.getElementsByTagName('th');
    for (i=0; i < hcols.length; i++) {
        //hcols[i].setAttribute('onclick', this.gizmoid+'.sort(this);');  
        hcols[i].onclick = u_stbl_sort_event_handler; //+ use bind? 
    }
};
Stbl.prototype.getSortFunction = function (col_type) {
    //+ bind only on first call? 
    if (col_type == "number") {
        return bind(this.rowCompareNumbers, this);
    } 
    else if (col_type == "float") {
        return bind(this.rowCompareFloats, this);
    } 
    else { // str 
        return bind(this.rowCompare, this); 
    }
}; 
Stbl.prototype.sort = function (th_dom) {
    var hcells = this.thead.getElementsByTagName('th');
    // get sortOn col index
    this.sortOn = -1; 
    for (var i=0; i<hcells.length; i++) {
        if (th_dom == hcells[i]) {
            this.sortOn = i;
            break;
        }
    }
    if (this.sortOn == -1) {
        return; 
    }
    var col_type = this.col_types[this.sortOn]; 
    var tbody = this.stable.getElementsByTagName('tbody')[0];
    var rows = tbody.getElementsByTagName('tr');
    this.num_rows = rows.length; 

    var reorderedRows = new Array();
    for (var i=0, length=rows.length; i<length; i++) {
        // remember current index, to be able to stabilize sort  
        reorderedRows[i] = [ rows[i].cloneNode(true), i ]; 
    }
    if (this.sortOn != this.prevSortOn) { 
        this.sortSense = "desc"; 
        sort_func = this.getSortFunction(col_type); 
        reorderedRows.sort(sort_func);        
        for (var i=0; i<reorderedRows.length; i++) {
            reorderedRows[i][0].className = (i % 2 == 0) ? "odd" : "even";  
            var cells = reorderedRows[i][0].getElementsByTagName('td');
            removeElementClass(cells[this.prevSortOn], "sorted");
            addElementClass(cells[this.sortOn], "sorted");
        }
    } 
    else {
        this.sortSense = (this.sortSense == "asc" ) ? "desc" : "asc";
        reorderedRows.reverse(); 
        for (var i=0; i<reorderedRows.length; i++) {
            reorderedRows[i][0].className = (i % 2 == 0) ? "odd" : "even";  
        }
    }
    if (this.onBeforeSort) {
        this.onBeforeSort(this);
    }
    removeElementClass(hcells[this.prevSortOn], "sorted");
    var spans = hcells[this.prevSortOn].getElementsByTagName('img');
    for (var j=0; j < spans.length; j++) {
        hcells[this.prevSortOn].removeChild(spans[j]);
    }
    this.prevSortOn = this.sortOn; 
    var newTbody = document.createElement('tbody');
    for (var i=0, length=reorderedRows.length; i<length; i++) {
        newTbody.appendChild(reorderedRows[i][0]);
    }
    this.stable.replaceChild(newTbody, tbody);
    u_stbl_setIndicator(th_dom, this.sortSense, this.base_assets); 
    if (this.onAfterSort) {
        this.onAfterSort(this.gizmoid, this.sortOn, this.sortSense);
    }
    this.handle_event('sort');
}; 
Stbl.prototype.rowCompare = function (a, b) {
    var aVal = a[0].getElementsByTagName('td')[this.sortOn].firstChild.nodeValue;
    var bVal = b[0].getElementsByTagName('td')[this.sortOn].firstChild.nodeValue;
    if (aVal == bVal) { return (a[1] - b[1]); }
    else { return (bVal > aVal ? 1 : -1 ); }
}; 
Stbl.prototype.rowCompareNumbers = function (a, b) {
    var aVal = parseInt(a[0].getElementsByTagName('td')[this.sortOn].firstChild.nodeValue);
    var bVal = parseInt(b[0].getElementsByTagName('td')[this.sortOn].firstChild.nodeValue);
    if (aVal == bVal) { return (a[1] - b[1]); }
    else { return (bVal - aVal); }
}; 
Stbl.prototype.rowCompareFloats = function (a, b) {
    var aVal = parseFloat(a[0].getElementsByTagName('td')[this.sortOn].firstChild.nodeValue);
    var bVal = parseFloat(b[0].getElementsByTagName('td')[this.sortOn].firstChild.nodeValue);
    if (aVal == bVal) { return (a[1] - b[1]); }
    else { return (bVal - aVal); }
};
Stbl.prototype.rowCompareDollars = function (a, b) {
    var aVal = parseFloat(a[0].getElementsByTagName('td')[this.sortOn].firstChild.nodeValue.substr(1));
    var bVal = parseFloat(b[0].getElementsByTagName('td')[this.sortOn].firstChild.nodeValue.substr(1));
    if (aVal == bVal) { return (a[1] - b[1]); }
    else { return (bVal - aVal); }
};

// ---------------------------------------------------------------------------
// SortableTable - unbound functions 

function u_stbl_sort_event_handler(ev)
{
    // get head col doc, table dom
    var n = null; 
    if (ev) { 
        var n = ev.target; // ev.currentTarget 
    }
    else { 
        var n = window.event.srcElement;
    } 
    if (n.nodeName != 'TH') {
        do {
            n = n.parentNode // n.parentElement
        }
        while (n.nodeName != 'TH')
    }
    var tbl = n; 
    do {
        tbl = tbl.parentNode 
    }
    while (tbl.nodeName != 'TABLE')
    
    // get js object for this sortable 
    var tbl_js = eval(tbl.id.substring(0, Math.floor(tbl.id.length/2)));

    // call sort
    if (!tbl_js.sorting) {    
        tbl_js.sorting = true;
        document.status = 'Sorting, please wait... ';
        tbl_js.sort(n);
        tbl_js.sorting = false;
    } 
    else {
        document.status = 'Already sorting, please wait... ';
    }
}

function u_stbl_setIndicator(e, sortSense, base_assets) 
{
    addElementClass(e, "sorted");
    var indicator = document.createElement("img");
    indicator.setAttribute("src", base_assets+sortSense+".gif");
    indicator.setAttribute("alt", (sortSense == "asc") ? "Ascending" : "Descending");
    addElementClass(indicator, "sortIndicator");
    e.appendChild(indicator);
}

function u_stbl_onAfterSort(gizmoid, sortOnColIndex, sortSense) {}

// ---------------------------------------------------------------------------


