'''
Gizmo(QP) Web Framework - http://gizmojo.org/
------------------------------------------------------------------------------
Copyright (C) 2007 Mario Ruggier, mario@ruggier.org
Licensed under the Open Software License version 3.0
'''
__url__ = "$URL: svn://gizmojo.org/pub/gz/trunk/lib/backup.py $"
__revision__ = "$Id: backup.py 964 2008-03-17 14:03:29Z mario $"


""" backup.py - backup versions of python source files.

   Usage: backup( dir, filetypes )       
   egs:   backup( '.', ['cpp', 'h'] )
   Version 1.3
   Author: Anand B Pillai 
   
   Modification Thu Apr 10 11:37:44 2003 
   No need to backup identical versions. Base verion is 1.0

   Tue Apr 22 18:15:11 2003 Made changes to copy backed-up files
                            to a directory named 'bak' in current
                            directory. Made function recursive.

   Sun Nov 30, 2003 (Mario Ruggier)
   - made types an optional parameter, with default value None meaning all types
   - added files parameter to initail (non-recursive) backup call, 
     to allow specifying files explicitly
   - removed variable "bakuppath" (giving error) and using "bakup" for copy()
"""

#

import sys, os
from shutil import copy
from string import lower
from filecmp import cmp

MAXVERSIONS=100

def backup(dir, types=None, files=None):
    "Back up files or files with extension in passed tuple types"

    if files is None:
        files=os.listdir(dir)

    # Backup directory
    for file in files:
        abspath = os.path.abspath(os.path.join(dir, file))

        if os.path.isfile(abspath):
            ext=lower((os.path.splitext(file))[1])[1:]
            if types is None or ext in types:
                # check for existence of previous versions
                index=1
                
                # create directory named 'bak' in current directory
                newdir = os.path.join(dir, 'bak')
                if not os.path.exists(newdir):
                    os.makedirs(newdir)
                
                while 1:
                    if index > MAXVERSIONS: 
                        break
                    bakup = os.path.join(newdir, file + '.bak.' + str(index))
                    if not os.path.exists(bakup): 
                        break
                    index += 1
                    
                if index>1:
                    # no need to backup if file and last version are identical
                    oldbakup = os.path.join(newdir, file + '.bak.' + str(index-1))
                    try:
                        if os.path.isfile(oldbakup) and cmp(abspath, oldbakup, shallow=0):
                            print 'File ', file, ': file is unchanged'
                            continue
                    except OSError, e:
                        pass

                print 'Backing up file \t', file ,' Version:', index
                try:
                    copy(abspath, bakup)
                except OSError, e:
                    pass
                
        elif os.path.isdir(abspath):
            backup(abspath, types)
            pass
