'''
Gizmo(QP) Web Framework - http://gizmojo.org/
------------------------------------------------------------------------------
Copyright (C) 2007 Mario Ruggier, mario@ruggier.org
Licensed under the Open Software License version 3.0
'''
__url__ = "$URL: svn://gizmojo.org/pub/gz/trunk/lib/evaluator.py $"
__revision__ = "$Id: evaluator.py 976 2008-03-19 11:51:34Z mario $"

import sys

class Evaluator(object):
    ''' Evaluator of code inside strings. Inspiration:
        http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/66018
        
    globals: the globals dict passed to eval()
    locals: the locals dict passed to eval
    errors: int:either(0:silent, 1:safe, 2:render, 3:raise)
    '''
    
    def __init__(self, globals, locals=None, errors=2):
        self.globals = globals
        self.locals = locals or sys._getframe(1).f_locals 
        self.errors = errors
        
    def __getitem__(self, name):
        try:
            return eval(name, self.globals, self.locals)
        except Exception, e:
            # NameError, AttributeError, SyntaxError
            return self.handle_get_error(name, e)

    def handle_get_error(self, name, e):
        errors = self.errors
        exc_indication = 'EVALERROR(%s)' % (name)
        exc_description = '%s: %s: %s' % (exc_indication, e.__class__.__name__, e)
        print exc_description # always log error 
        if not errors: # silent
            return '' 
        elif errors == 3: # raise
            raise e.__class__('%s: %s' % (exc_indication, e))
        elif errors == 2: # render
            return ' [%s] ' % (exc_description)
        elif errors == 1: # safe
            return ' [%s] ' % (exc_indication)

    def set_locals(self, locals):
        # Allows easy "locals"-ization (and re-using) of Evaluator instance.
        # Globals remain unchanged.
        self.locals = locals


class EvaluatorRecursive(Evaluator):
        
    def __getitem__(self, name):
        try:
            name = name % self # e.g. %(obj.%(method)s())s
            return eval(name, self.globals, self.locals)
        except Exception, e:
            return self.handle_get_error(name, e)

