'''
Gizmo(QP) Web Framework - http://gizmojo.org/
------------------------------------------------------------------------------
Copyright (C) 2007 Mario Ruggier, mario@ruggier.org
Licensed under the Open Software License version 3.0
'''
__url__ = "$URL: svn://gizmojo.org/pub/gz/trunk/pub/durus_publisher.py $"
__revision__ = "$Id: durus_publisher.py 964 2008-03-17 14:03:29Z mario $"

import socket
from durus.btree import BTree
import qp.pub.publish
from gz.pub.common import get_user, get_info
from gz.gizmo.gspec import match
from publisher import Publisher

class DurusPublisher(Publisher, qp.pub.publish.DurusPublisher):
    
    # assert get_site().get_durus_address() == True
    # otherwise: DurusPublisher.__init__(site) 
    def __init__(self, connection=None, site=None, **kw):
        assert connection is not None
        self.connection = connection
        Publisher.__init__(self, site=site, **kw)
        self.ensure_database_initialized()

    # 
    
    def process_hit(self, hit, attempt=0):
        ''' Ensure that the durus server is up, and that this publisher's 
        connection is good. To trigger the potential exception, we do a
        systematic abort() at the start of each request -- that seems to be 
        anyway good Durus practice. 
        '''
        if attempt < 3: 
            try:
                self.get_connection().abort()
            except socket.error, (code,message) :
                if code == 54:
                    print ('Connection reset by peer... '
                           'restarting durus server.')
                    self.site.stop_durus()
                    self.site.start_durus()
                if code in (32,54):
                    print 'Broken pipe... resetting durus connection.'
                    durus_address = self.site.get_durus_address()
                    if durus_address:
                        durus_host, durus_port = durus_address
                        from durus.client_storage import ClientStorage
                        from durus.connection import Connection
                        self.connection = Connection(
                            ClientStorage(port=durus_port, host=durus_host),
                            cache_size=self.site.get_durus_cache_size() )
                self.log_exception()
                attempt += 1
                self.process_hit(hit, attempt)
        super(DurusPublisher, self).process_hit(hit)

    # additional db methods
    # These methods enable a certain amount of reuse of applictaion code,
    # irrespective of which data model is being used. 
    
    def get_user_class(self):
        from qp.pub.user import User
        return User

    def get_session_class(self):
        from qp.pub.session import Session
        return Session

    def match(self, value, cls, attr):
        return match(value, getattr(cls, attr+'_is'))
        
    # Authentication methods

    def get_session(self):
        return self.get_hit().get_session()

    def get_user(self):
        return self.get_session().get_effective_user()

    def get_default_realm(self):
        if self.get('realms') is not None:
            return self.get('realms')[0]
        else:
            return self.site.get_name()

    def ensure_signed_in(self, **kw):
        if not get_user():
            if not self.lock_responding_now():
                get_info()['respond_now_qcomp'] = (self.get_root_directory().login, '')
                self.ensure_signed_in_using_form(**kw)

    def ensure_signed_in_using_form(self, title='Please Sign In',
                                    realm=None, **kw):
        '''
        Make sure that the current user is signed in.
        This presents a form to the user.  Because the form transmits the
        password, this redirects to the https address before presenting the
        form. The realm, if given, identifies which of the user's passwords
        should be used.
        '''
        if not get_user():
            self.secure()
            hit = self.get_hit()
            request, session = hit.get_request(), hit.get_session()
            login_dir = self.get_root_directory().login
            back_to = request.get_field('back_to') or ""
            current_url = request.get_path_query()
            if current_url.find(self.get_url_path_for('login'))!=-1:
                back_to = '/'
            elif self.has_key_path('url_paths', 'register'):
                if current_url.find(self.get_url_path_for('register'))!=-1:
                    back_to = '/'
            form = login_dir.login_form()
            form.add_hidden('back_to', value=back_to)
            if form.is_submitted() and not form.has_errors():
                name = form.get('name') or ''
                passwd = form.get('password') or ''
                user = self.get_users().get(name.lower())
                if realm is None:
                    realm = self.get_default_realm()
                if user and user.has_password(passwd, realm=realm):
                    session.set_authenticated(user)
                    self.redirect(back_to)
                    
            login_dir.respond_login_form(title, form, status=403, **kw)

    #
