'''
Gizmo(QP) Web Framework - http://gizmojo.org/
------------------------------------------------------------------------------
Copyright (C) 2007 Mario Ruggier, mario@ruggier.org
Licensed under the Open Software License version 3.0
'''
__url__ = "$URL: svn://gizmojo.org/pub/gz/trunk/skin/evoque_templating.py $"
__revision__ = "$Id: evoque_templating.py 979 2008-03-19 14:33:16Z mario $"

from qpy import h8
from evoque.domain import Domain
from gz.pub.common import get_hit, get_dhi
from skin import Skin
import render
import html
import gizmo
import site
import page


class EvoqueTemplating(Skin):
    """ 
    Should provide:
    - page() delegator method, to return a formatted page according to 
      the selected template.
    """
    
    def __init__(self, doctype="XHTML_10_Transitional"):
        super(EvoqueTemplating, self).__init__(doctype=doctype)
        # domain must be set after init (to conform to Skin API)
        self.domain = None
        # aliases for (default collection) template names
        self.aliases = {} 
        
    def set_domain(self, default_dir, **kw):
        self.domain = Domain(default_dir, **kw)
    
    def setup_globals(self):
        self.domain.set_namespace_on_globals("html", html)
        self.domain.set_namespace_on_globals("gizmo", gizmo)
        self.domain.set_namespace_on_globals("site", site)
        self.domain.set_namespace_on_globals("page", page)
    
    def unalias_name(self, name):
        """ (name:str) -> str
        If name is an alias, then use the referenced name, itherwise use name.
        """
        return self.aliases.get(name, name)

    def page(self, title, content=(), locals=None, template="", src=None, 
                collection=None, raw=False, quoting=None, input_encoding=None, 
                **kw):
        """ Wrap for: self.domain.get_template().evoque() -> h8 """
        if isinstance(content, basestring):
            content = (content,)
        content = h8.from_list(content)
        collection = self.domain.get_collection(collection)
        if collection is self.domain.get_collection():
            template = self.unalias_name(template)
        t = self.domain.get_template(template, 
                src, collection, raw, None, quoting, input_encoding)
        self.setdefault_dhi(kw) # ensures kw["dhi"] for render.dhi_page()
        return render.dhi_page(title, 
                t.evoque(locals, raw=raw, quoting=quoting, title=title, 
                        content=content, hit=get_hit(), **kw), 
                doctype=self.doctype, 
                **kw)
    
