'''
Gizmo(QP) Web Framework - http://gizmojo.org/
------------------------------------------------------------------------------
Copyright (C) 2007 Mario Ruggier, mario@ruggier.org
Licensed under the Open Software License version 3.0
'''
__url__ = "$URL: svn://gizmojo.org/pub/gz/trunk/skin/qpy_skin.py $"
__revision__ = "$Id: qpy_skin.py 969 2008-03-18 18:39:49Z mario $"

from qpy import h8
from gz.pub.common import get_publisher
from skin import Skin
import render

class QpySkin(Skin):
    """ QP Style Skin Class
    
    Generation of a page is via delegation to a page renderer method, 
    selected via the 'template' kwarg specified on page().
    """
    
    def __init__(self, doctype='XHTML_10_Transitional'):
        super(QpySkin, self).__init__(doctype=doctype)
        self.templates = {}
        self.init_templates()
    
    def init_templates(self):
        """ QP Style templates are page() methods. 
        Declare template names, and the page() method to be used. 
        Template for None is considered as the default template. 
        """
        self.templates[None] = self.std_page
        self.templates['std'] = self.std_page
        self.templates['durus'] = self.std_page
        self.templates['login'] = self.register_login_page
        self.templates['register'] = self.register_login_page
    
    def page(self, title, *content, **kw):
        """ (title:str, *content, **kw) -> h8 
        Page dispatcher method. 
        """
        kw.setdefault("doctype", self.doctype)
        return self.get_template(kw.get('template'))(title, *content, **kw)

    def get_template(self, name):
        """ (name:str) -> function
        Returns a page() function, to be called to generate the page.
        """
        return self.templates.get(name) or self.templates[None]

    # ------------------------------------------------------------------

    def std_page(self, title, *content, **kw):
        # (title, *content, **kw) -> h8 
        dhi = self.setdefault_dhi(kw)
        return render.dhi_page(title, *content, **kw)

    def register_login_page(self, title, *content, **kw):
        # (title, *content, **kw) -> h8 
        publisher = get_publisher()
        if publisher.has_key_path('url_paths', 'register'):
            content += (h8(
                '<div>'+self.format_link_register_login_logout()+'</div>'),)
        else:
            content += (h8('<div>'+self.format_link_login_logout()+'</div>'),)
        return self.std_page(title, *content, **kw)
    
