'''
Gizmo(QP) Web Framework - http://gizmojo.org/
------------------------------------------------------------------------------
Copyright (C) 2007 Mario Ruggier, mario@ruggier.org
Licensed under the Open Software License version 3.0
'''
__url__ = "$URL: svn://gizmojo.org/pub/gz/trunk/skin/skin.py $"
__revision__ = "$Id: skin.py 968 2008-03-18 09:31:43Z mario $"

from gz.fill import doctypes
from gz.pub.common import get_dhi

class Skin(object):
    """ Abstract Skin class

    A Skin is a manager of a related set of templates -- 
    a skin defines *page templates*, in accordance with its own meaning 
    of what a page template is, and manages their invocation. To respond, 
    an export need only call common.page(), with whatever parameters 
    necessary to select the right page template, that will invoke 
    Skin.page() to delegate to the selected template, 
    that will in turn generate the response body.

    Thus, a Skin class must provide:
    - page() delegator method, to return a formatted page according to 
      the selected template.
    """

    def __init__(self, doctype="XHTML_10_Transitional"):
        self.doctype = doctypes.doctype(doctype) # actual doctype h8 string
    
    def page(self, title, *content, **kw):
        ''' (title:str, *content, **kw) -> h8 
        Returns a formatted page. Sub-classes must specify.
        '''
        raise NotImplementedError
    
    def setdefault_dhi(self, kw):
        ''' (kw:dict) -> dhi
        Ensure a kw['dhi'] is set. If not set, set it with result from 
        get_dhi() -- implies that this will SET the dhi for this hit if it 
        was not previously set. 
        '''
        if kw.has_key('dhi'):
            return kw['dhi'] 
        return kw.setdefault('dhi', get_dhi())

